@@ -446,7 +446,7 @@ match_DICTIONARY (const char *restrict filename,
 
   for (i = 0; i < _obj->numitems; i++)
     {
-      MATCH_OBJECT (DICTIONARY, text[i], 3);
+      MATCH_OBJECT (DICTIONARY, texts[i], 3);
     }
   return found;
 }
@@ -557,7 +557,7 @@ match_LAYER_INDEX (const char *restrict filename,
 
   for (i = 0; i < _obj->num_entries; i++)
     {
-      MATCH_OBJECT (LAYER_INDEX, entries[i].layer, 8);
+      MATCH_OBJECT (LAYER_INDEX, entries[i].layername, 8);
     }
   return found;
 }
@@ -834,6 +834,7 @@ match_ASSOCACTION (const char *restrict filename,
   MATCH_OBJECT (ASSOCACTION, body.expression, 0);
   return found;
 }
+
 static int
 match_ASSOCOSNAPPOINTREFACTIONPARAM (const char *restrict filename,
                                      const Dwg_Object *restrict obj)
@@ -845,15 +846,16 @@ match_ASSOCOSNAPPOINTREFACTIONPARAM (const char *restrict filename,
   MATCH_OBJECT (ASSOCOSNAPPOINTREFACTIONPARAM, name, 1);
   return found;
 }
+
 static int
-match_ACDBNAVISWORKSMODELDEF (const char *restrict filename,
-                              const Dwg_Object *restrict obj)
+match_NAVISWORKSMODELDEF (const char *restrict filename,
+                          const Dwg_Object *restrict obj)
 {
   char *text;
   int found = 0;
-  const Dwg_Object_ACDBNAVISWORKSMODELDEF *_obj
-      = obj->tio.object->tio.ACDBNAVISWORKSMODELDEF;
-  MATCH_OBJECT (ACDBNAVISWORKSMODELDEF, path, 1);
+  const Dwg_Object_NAVISWORKSMODELDEF *_obj
+      = obj->tio.object->tio.NAVISWORKSMODELDEF;
+  MATCH_OBJECT (NAVISWORKSMODELDEF, path, 1);
   return found;
 }
 
@@ -999,7 +1001,8 @@ match_BLOCK_HEADER (const char *restrict filename,
                 found += match_LTYPE (filename, obj);
               else if (obj->type == DWG_TYPE_DIMSTYLE)
                 found += match_DIMSTYLE (filename, obj);
-              else if (obj->fixedtype == DWG_TYPE_DICTIONARY)
+              else if (obj->fixedtype == DWG_TYPE_DICTIONARY
+                       || obj->fixedtype == DWG_TYPE_DICTIONARYWDFLT)
                 found += match_DICTIONARY (filename, obj);
 
               else if (obj->fixedtype == DWG_TYPE_GROUP)
@@ -1049,8 +1052,8 @@ match_BLOCK_HEADER (const char *restrict filename,
               else if (obj->fixedtype
                        == DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM)
                 found += match_ASSOCOSNAPPOINTREFACTIONPARAM (filename, obj);
-              else if (obj->fixedtype == DWG_TYPE_ACDBNAVISWORKSMODELDEF)
-                found += match_ACDBNAVISWORKSMODELDEF (filename, obj);
+              else if (obj->fixedtype == DWG_TYPE_NAVISWORKSMODELDEF)
+                found += match_NAVISWORKSMODELDEF (filename, obj);
             }
         }
       if (!opt_text)
@@ -1229,6 +1232,7 @@ main (int argc, char *argv[])
                            NULL
 #    endif
   );
+  free (pattern16);
   if (errcode != 0 && errcode != 100)
     {
       pcre2_get_error_message_8 (errcode, (PCRE2_UCHAR8 *)buf, 4096);
@@ -1261,8 +1265,7 @@ main (int argc, char *argv[])
       count += match_BLOCK_HEADER (filename, dwg_model_space_ref (&dwg));
       for (k = 0; k < dwg.block_control.num_entries; k++)
         {
-          count += match_BLOCK_HEADER (filename,
-                                       dwg.block_control.block_headers[k]);
+          count += match_BLOCK_HEADER (filename, dwg.block_control.entries[k]);
         }
       count += match_BLOCK_HEADER (filename, dwg_paper_space_ref (&dwg));
 
