@@ -94,13 +94,14 @@ bit_set_position (Bit_Chain *dat, unsigned long bitpos)
                  dat->byte, dat->size)
     }
 }
+
 /* Set the chain so that dat->byte starts at 0 */
 void
 bit_reset_chain (Bit_Chain *dat)
 {
   unsigned long pos = dat->byte;
   dat->byte = 0;
-  if (dat->byte <= dat->size) // not already overflowed
+  if (pos < dat->size) // not already overflowed
     dat->chain += pos;
   if (dat->size > 0)
     dat->size -= pos;
@@ -255,20 +256,22 @@ bit_write_3B (Bit_Chain *dat, unsigned char value)
     }
 }
 
-/** Read 1 nibble.
+/** Read 4 bits.
  */
 BITCODE_4BITS
 bit_read_4BITS (Bit_Chain *dat)
 {
   // clang-format off
-  return bit_read_B (dat) << 4 |
+  BITCODE_4BITS b =
+         bit_read_B (dat) << 3 |
          bit_read_B (dat) << 2 |
          bit_read_B (dat) << 1 |
          bit_read_B (dat);
   // clang-format on
+  return b;
 }
 
-/** Write 1 nibble.
+/** Write 4 bits.
  */
 void
 bit_write_4BITS (Bit_Chain *dat, unsigned char value)
@@ -1127,7 +1130,6 @@ bit_write_BT (Bit_Chain *dat, double value)
 int
 bit_read_H (Bit_Chain *restrict dat, Dwg_Handle *restrict handle)
 {
-  unsigned char *restrict val;
   int i;
 
   handle->code = bit_read_RC (dat);
@@ -1135,7 +1137,7 @@ bit_read_H (Bit_Chain *restrict dat, Dwg_Handle *restrict handle)
   handle->code = (handle->code & 0xf0) >> 4;
 
   handle->value = 0;
-  if (handle->size > sizeof(val) || handle->code > 14)
+  if (handle->size > sizeof(BITCODE_RC *) || handle->code > 14)
     {
       loglevel = dat->opts & 0xf;
       LOG_WARN ("Invalid handle-reference, longer than 8 bytes: " FORMAT_H,
@@ -1144,9 +1146,28 @@ bit_read_H (Bit_Chain *restrict dat, Dwg_Handle *restrict handle)
     }
 
   // TODO: little-endian only
-  val = (unsigned char *)&handle->value;
-  for (i = handle->size - 1; i >= 0; i--)
-    val[i] = bit_read_RC (dat);
+  // x86_64 gcc-9 miscompilation with val[i]: (%rbx) being dat+1
+#if defined(__GNUC__) && (__GNUC__ == 9) && (__GNUC_MINOR__ <= 2) \
+  && (SIZEOF_SIZE_T == 8)                                         \
+  && (defined(__x86_64__) || defined(__x86_64) || defined(_M_AMD64) || defined(_M_X64))
+  {
+#warning x86_64 gcc-9.2 codegen is seriously broken! better use an older version
+    BITCODE_RC *restrict val;
+    val = (BITCODE_RC *)&(handle->value);
+    for (i = handle->size - 1; i >= 0; i--)
+      {
+        BITCODE_RC c = bit_read_RC (dat);
+        val[i] = c;
+      }
+  }
+#else
+  {
+    BITCODE_RC *restrict val;
+    val = (BITCODE_RC *)&(handle->value);
+    for (i = handle->size - 1; i >= 0; i--)
+      val[i] = bit_read_RC (dat);
+  }
+#endif
 
   return 0;
 }
@@ -1456,6 +1477,25 @@ bit_convert_TU (BITCODE_TU restrict wstr)
 
   if (!wstr)
     return NULL;
+#ifdef HAVE_ALIGNED_ACCESS_REQUIRED
+  /* access bytewise. cannot copy to heap yet, because we have no length */
+  if ((uintptr_t)wstr % SIZEOF_SIZE_T)
+    {
+      unsigned char *b = (unsigned char*)wstr;
+      c = (b[0] << 8) + b[1];
+      while (c)
+        {
+          len++;
+          if (c >= 256)
+            len++;
+          if (c >= 0x800)
+            len++;
+          b += 2;
+          c = (b[0] << 8) + b[1];
+        }
+    }
+  else
+#endif
   while ((c = *tmp++))
     {
       len++;
@@ -1470,6 +1510,35 @@ bit_convert_TU (BITCODE_TU restrict wstr)
     }
   str = malloc (len + 1);
   i = 0;
+#ifdef HAVE_ALIGNED_ACCESS_REQUIRED
+  if ((uintptr_t)wstr % SIZEOF_SIZE_T)
+    {
+      unsigned char *b = (unsigned char*)wstr;
+      c = (b[0] << 8) + b[1];
+      while (c)
+        {
+          if (c < 256)
+            {
+              str[i++] = c & 0xFF;
+            }
+          else if (c < 0x800)
+            {
+              str[i++] = (c >> 6) | 0xC0;
+              str[i++] = (c & 0x3F) | 0x80;
+            }
+          else /* if (c < 0x10000) */
+            {
+              str[i++] = (c >> 12) | 0xE0;
+              str[i++] = ((c >> 6) & 0x3F) | 0x80;
+              str[i++] = (c & 0x3F) | 0x80;
+            }
+
+          b += 2;
+          c = (b[0] << 8) + b[1];
+        }
+    }
+  else
+#endif
   while ((c = *wstr++))
     {
       if (c < 256)
